-- EuOS Kernel: root filesystem
-- by Jeffrey Fielding (JJProg@cyberbury.net)
-- now read returns -1 on EOF, not {} (so as not to be confused with
-- no data available on sockets etc.)
include file.e
include misc.e
sequence k_rootfs_intfds, k_rootfs_extfds, k_rootfs_dev
integer k_rootfs_handler
k_rootfs_intfds = {}
k_rootfs_extfds = {}
function k_rootfs_fix(sequence s)
	if k_rootfs_dev[length(k_rootfs_dev)] = '/' then
		s = k_rootfs_dev & s[2..length(s)]
	else
		s = k_rootfs_dev & s
	end if
	if platform() != LINUX then
		for i = 1 to length(s) do
			if s[i] = '/' then
				s[i] = '\\'
			end if
		end for
	end if
	return s
end function
procedure k_rootfs_onmount(sequence dev, sequence root)
	puts(1,"Mounted rootfs on " & root & " from " & dev & "\n")
	k_rootfs_dev = dev
end procedure
procedure k_rootfs_onunmount(sequence root)
	puts(1,"unmounting " & root & "\n")
end procedure
function k_rootfs_close(integer fd)
	integer i
	i = find(fd,k_rootfs_intfds)
	k_rootfs_intfds[i] = -1
	k_freefd(fd)
	close(k_rootfs_extfds[i])
	return 0
end function
function k_rootfs_read(integer fd, integer n)
	integer i, c, f
	sequence r
	i = find(fd,k_rootfs_intfds)
	f = k_rootfs_extfds[i]
	r = {}
	for j = 1 to n do
		c = getc(f)
		if c = -1 then
			exit
		else
			r &= c
		end if
	end for
	if length(r) = 0 then
		return -1
	end if
	return r
end function
function k_rootfs_write(integer fd, object data)
	puts(k_rootfs_extfds[find(fd,k_rootfs_intfds)],data)
	return 0
end function
function k_rootfs_seek(integer fd, atom where)
	return seek(k_rootfs_extfds[find(fd,k_rootfs_intfds)],where)
end function
function k_rootfs_tell(integer fd)
	return where(k_rootfs_extfds[find(fd,k_rootfs_intfds)])
end function
function k_rootfs_ioctl(integer fd, integer op, object x)
	return 0
end function
function k_rootfs_open(sequence file, sequence mode)
	integer fd, i, j, f
	file = k_rootfs_fix(file)
	f = open(file,mode)
	if f = -1 then
		return -1
	end if
	fd = k_allocfd(k_rootfs_handler)
	i = find(-1,k_rootfs_intfds)
	if i then
		k_rootfs_intfds[i] = fd
		k_rootfs_extfds[i] = f
	else
		k_rootfs_intfds &= fd
		k_rootfs_extfds &= f
	end if
	return fd
end function
function k_rootfs_dir(sequence f)
	return dir(k_rootfs_fix(f))
end function
function k_rootfs_del(sequence f)
	if platform() = LINUX then
		system("rm -f " & k_rootfs_fix(f),2)
	else
		system("del " & k_rootfs_fix(f),2)
	end if
	return 0
end function
function k_rootfs_mkdir(sequence f)
	system("mkdir " & k_rootfs_fix(f),2)
	return 0
end function
k_rootfs_handler = k_new_fd_handler({routine_id("k_rootfs_close"),
				    routine_id("k_rootfs_read"),
				    routine_id("k_rootfs_write"),
				    routine_id("k_rootfs_seek"),
				    routine_id("k_rootfs_tell"),
				    routine_id("k_rootfs_ioctl")})
k_register_fs_type("rootfs",{routine_id("k_rootfs_open"),
			    routine_id("k_rootfs_dir"),
			    routine_id("k_rootfs_del"),
			    routine_id("k_rootfs_mkdir"),
			    routine_id("k_rootfs_onmount"),
			    routine_id("k_rootfs_onunmount")})
k_mount(".","/","rootfs")


