-- Simple (cooperative) multitasking for EuOS
-- By Jeffrey Fielding (JJProg@cyberbury.net)
-- 1.9: BUGFIX - set last error now uses correct task data ID
-- 1.7: Added parent task id to data
-- 1.5: BUGFIX - now correctly verifies pids in getTaskData and setTaskData
-- new in 1.3: errors, and added error checking
-- new in 1.1: k_task_data
-- task_data format:
-- {task specific data, command line, stdin, stdout, stderr, last error, parent
-- task}
constant DEFAULT_DATA = {0,{},0,1,2,0,0}
constant MAX_DATA = length(DEFAULT_DATA)
sequence k_task_ids, k_task_priorities, k_task_data, k_task_joined
integer k_current_pid
k_task_ids = {}
k_task_priorities = {}
k_current_pid = 0
k_task_data = {}
k_task_joined = {}
global procedure k_setLastError(integer id, sequence message)
	if k_current_pid then
		k_task_data[k_current_pid][6] = {id, message}
		printf(1,"Error %d: %s\n",{id,message})
	end if
end procedure
global function k_getLastError()
	if k_current_pid then
		return k_task_data[k_current_pid][6]
	else
		return 0
	end if
end function
global function k_checkError()
	if k_current_pid then
		return sequence(k_task_data[k_current_pid][6])
	else
		return 0
	end if
end function
global procedure k_clearError()
	if k_current_pid then
		k_task_data[k_current_pid][6] = 0
	end if
end procedure
global function k_getPid()
	return k_current_pid
end function
function k_verifyTask(integer id)
	if id > 0 and id <= length(k_task_ids) then
		if k_task_ids[id] != -1 then
			return 1
		end if
	end if
	k_setLastError(1,"Invalid task id.")
	return 0
end function
function k_verifyTask2(integer id)
	if id > 0 and id <= length(k_task_ids) then
		return 1
	end if
	k_setLastError(1,"Invalid task id.")
	return 0
end function
global procedure k_join(integer pid)
	if k_verifyTask(pid) then
		k_task_joined[pid] &= k_current_pid
		k_task_priorities[k_current_pid] = 0
	end if
end procedure
global procedure k_killTask(integer id)
	if k_verifyTask2(id) then
		if k_task_ids[id] != -1 then
			k_task_ids[id] = -1
			k_task_priorities[id] = 0
			for i = 1 to length(k_task_joined[id]) do
				k_task_priorities[k_task_joined[id][i]] = 1
			end for
			k_task_joined[id] = {}
		end if
	end if
end procedure
global procedure k_killAll()
	for i = 1 to length(k_task_ids) do
		if k_task_ids[i] != -1 then
			k_killTask(i)
		end if
	end for
end procedure
global function k_newTask(integer id)
	integer index
	sequence data
	if k_current_pid = 0 then
		data = DEFAULT_DATA
	else
		data = k_task_data[k_current_pid]
		data[1] = 0
	end if
	data[7] = k_current_pid
	index = find(-1,k_task_ids)
	if index then
		k_task_ids[index] = id
		k_task_priorities[index] = 1
		k_task_data[index] = data
		return index
	else
		k_task_ids &= id
		k_task_priorities &= 1
		k_task_data = append(k_task_data,data)
		k_task_joined = append(k_task_joined,{})
		return length(k_task_ids)
	end if
end function
global procedure k_setCommandLine(integer id, sequence cl)
	if k_verifyTask(id) then
		k_task_data[id][2] = cl
	end if
end procedure
global function k_getCommandLine()
	if k_current_pid then
		return k_task_data[k_current_pid][2]
	else
		return {}
	end if
end function
function k_verifyData(integer id)
	if id > 0 and id <= MAX_DATA then
		return 1
	else
		k_setLastError(2,"Invalid data index.")
		return 0
	end if
end function
global procedure k_setTaskData(integer pid, integer id, object val)
	if k_verifyTask(pid) then
		if k_verifyData(id) then
			k_task_data[pid][id] = val
		end if
	end if
end procedure
global function k_getTaskData(integer pid, integer id)
	if k_verifyTask(pid) then
		if k_verifyData(id) then
			return k_task_data[pid][id]
		end if
	end if
	return -1
end function
global procedure k_setPriority(integer id, integer priority)
	if k_verifyTask(id) then
		k_task_priorities[id] = priority
	end if
end procedure
global procedure k_startMultitasking()
	integer tasks_run
	integer x
	tasks_run = 1
	while tasks_run do
		tasks_run = 0
		for i = 1 to length(k_task_priorities) do
			if k_task_priorities[i] then
				tasks_run += 1
				k_current_pid = i
				x = call_func(k_task_ids[i],{})
				if x = -1 then
					k_killTask(i)
				else
					k_task_ids[i] = x
				end if
			end if
		end for
	end while
end procedure






